@@ -6,13 +6,13 @@
 #include "libbb.h"
 #include "unarchive.h"
 
-void FAST_FUNC seek_by_jump(const archive_handle_t *archive_handle, unsigned amount)
+void FAST_FUNC seek_by_jump(int fd, off_t amount)
 {
 	if (amount
-	 && lseek(archive_handle->src_fd, (off_t) amount, SEEK_CUR) == (off_t) -1
+	 && lseek(fd, amount, SEEK_CUR) == (off_t) -1
 	) {
 		if (errno == ESPIPE)
-			seek_by_read(archive_handle, amount);
+			seek_by_read(fd, amount);
 		else
 			bb_perror_msg_and_die("seek failure");
 	}
