@@ -8,8 +8,6 @@
  *
  * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
  */
-
-//#include <sys/ioctl.h>
 #include "libbb.h"
 
 /* From <linux/kd.h> */
@@ -23,7 +21,7 @@ enum {
 int setkeycodes_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int setkeycodes_main(int argc, char **argv)
 {
-	int fd, sc;
+	int fd;
 	struct kbkeycode a;
 
 	if (!(argc & 1) /* if even */ || argc < 2) {
@@ -32,17 +30,17 @@ int setkeycodes_main(int argc, char **argv)
 
 	fd = get_console_fd_or_die();
 
-	while (argc > 2) {
-		a.keycode = xatou_range(argv[2], 0, 127);
-		a.scancode = sc = xstrtoul_range(argv[1], 16, 0, 255);
-		if (a.scancode > 127) {
-			a.scancode -= 0xe000;
-			a.scancode += 128;
+	while (argv[1]) {
+		int sc = xstrtoul_range(argv[1], 16, 0, 0xe07f);
+		if (sc >= 0xe000) {
+			sc -= 0xe000;
+			sc += 0x0080;
 		}
+		a.scancode = sc;
+		a.keycode = xatou_range(argv[2], 0, 255);
 		ioctl_or_perror_and_die(fd, KDSETKEYCODE, &a,
 			"can't set SCANCODE %x to KEYCODE %d",
 			sc, a.keycode);
-		argc -= 2;
 		argv += 2;
 	}
 	return EXIT_SUCCESS;
