@@ -10,23 +10,12 @@
 /* BB_AUDIT SUSv3 (virtually) compliant -- uses nicer GNU format for -l. */
 /* http://www.opengroup.org/onlinepubs/007904975/utilities/cmp.html */
 
-/* Mar 16, 2003      Manuel Novoa III   (mjn3@codepoet.org)
- *
- * Original version majorly reworked for SUSv3 compliance, bug fixes, and
- * size optimizations.  Changes include:
- * 1) Now correctly distinguishes between errors and actual file differences.
- * 2) Proper handling of '-' args.
- * 3) Actual error checking of i/o.
- * 4) Accept SUSv3 -l option.  Note that we use the slightly nicer gnu format
- *    in the '-l' case.
- */
-
 #include "libbb.h"
 
 static const char fmt_eof[] ALIGN1 = "cmp: EOF on %s\n";
-static const char fmt_differ[] ALIGN1 = "%s %s differ: char %"OFF_FMT"d, line %d\n";
-// This fmt_l_opt uses gnu-isms.  SUSv3 would be "%.0s%.0s%"OFF_FMT"d %o %o\n"
-static const char fmt_l_opt[] ALIGN1 = "%.0s%.0s%"OFF_FMT"d %3o %3o\n";
+static const char fmt_differ[] ALIGN1 = "%s %s differ: char %"OFF_FMT"u, line %u\n";
+// This fmt_l_opt uses gnu-isms.  SUSv3 would be "%.0s%.0s%"OFF_FMT"u %o %o\n"
+static const char fmt_l_opt[] ALIGN1 = "%.0s%.0s%"OFF_FMT"u %3o %3o\n";
 
 static const char opt_chars[] ALIGN1 = "sl";
 #define CMP_OPT_s (1<<0)
@@ -37,8 +26,7 @@ int cmp_main(int argc UNUSED_PARAM, char **argv)
 {
 	FILE *fp1, *fp2, *outfile = stdout;
 	const char *filename1, *filename2 = "-";
-	USE_DESKTOP(off_t skip1 = 0, skip2 = 0;)
-	off_t char_pos = 0;
+	off_t skip1 = 0, skip2 = 0, char_pos = 0;
 	int line_pos = 1; /* Hopefully won't overflow... */
 	const char *fmt;
 	int c1, c2;
@@ -48,8 +36,8 @@ int cmp_main(int argc UNUSED_PARAM, char **argv)
 	xfunc_error_retval = 2;	/* 1 is returned if files are different. */
 
 	opt_complementary = "-1"
-			USE_DESKTOP(":?4")
-			SKIP_DESKTOP(":?2")
+			IF_DESKTOP(":?4")
+			IF_NOT_DESKTOP(":?2")
 			":l--s:s--l";
 	opt = getopt32(argv, opt_chars);
 	argv += optind;
@@ -59,14 +47,12 @@ int cmp_main(int argc UNUSED_PARAM, char **argv)
 
 	if (*++argv) {
 		filename2 = *argv;
-#if ENABLE_DESKTOP
-		if (*++argv) {
+		if (ENABLE_DESKTOP && *++argv) {
 			skip1 = XATOOFF(*argv);
 			if (*++argv) {
 				skip2 = XATOOFF(*argv);
 			}
 		}
-#endif
 	}
 
 	fp2 = xfopen_stdin(filename2);
@@ -83,10 +69,10 @@ int cmp_main(int argc UNUSED_PARAM, char **argv)
 	else
 		fmt = fmt_differ;
 
-#if ENABLE_DESKTOP
-	while (skip1) { getc(fp1); skip1--; }
-	while (skip2) { getc(fp2); skip2--; }
-#endif
+	if (ENABLE_DESKTOP) {
+		while (skip1) { getc(fp1); skip1--; }
+		while (skip2) { getc(fp2); skip2--; }
+	}
 	do {
 		c1 = getc(fp1);
 		c2 = getc(fp2);
@@ -108,7 +94,7 @@ int cmp_main(int argc UNUSED_PARAM, char **argv)
 				outfile = stderr;
 				/* There may have been output to stdout (option -l), so
 				 * make sure we fflush before writing to stderr. */
-				xfflush_stdout();
+				fflush_all();
 			}
 			if (!(opt & CMP_OPT_s)) {
 				if (opt & CMP_OPT_l) {
