@@ -101,7 +101,7 @@ DECLARE_STR_CONV(int, i, u)
 
 /* Specialized */
 
-int BUG_xatou32_unimplemented(void);
+uint32_t BUG_xatou32_unimplemented(void);
 static ALWAYS_INLINE uint32_t xatou32(const char *numstr)
 {
 	if (UINT_MAX == 0xffffffff)
@@ -158,7 +158,7 @@ unsigned bb_strtou(const char *arg, char **endp, int base) FAST_FUNC;
 int bb_strtoi(const char *arg, char **endp, int base) FAST_FUNC;
 #endif
 
-int BUG_bb_strtou32_unimplemented(void);
+uint32_t BUG_bb_strtou32_unimplemented(void);
 static ALWAYS_INLINE
 uint32_t bb_strtou32(const char *arg, char **endp, int base)
 {
