@@ -2,14 +2,14 @@
 /*
  * Utility routines.
  *
- * Copyright (C) 2007 Denis Vlasenko
+ * Copyright (C) 2007 Denys Vlasenko
  *
  * Licensed under GPL version 2, see file LICENSE in this tarball for details.
  */
 
 #include "libbb.h"
 
-const char *bb_basename(const char *name)
+const char* FAST_FUNC bb_basename(const char *name)
 {
 	const char *cp = strrchr(name, '/');
 	if (cp)
