@@ -1,6 +1,6 @@
 /* vi: set sw=4 ts=4: */
 /*
- * Ckeck user and group names for illegal characters
+ * Check user and group names for illegal characters
  *
  * Copyright (C) 2008 Tito Ragusa <farmatito@tiscali.it>
  *
@@ -18,16 +18,20 @@
 
 void FAST_FUNC die_if_bad_username(const char *name)
 {
-	goto skip; /* 1st char being dash isn't valid */
+	/* 1st char being dash or dot isn't valid: */
+	goto skip;
+	/* For example, name like ".." can make adduser
+	 * chown "/home/.." recursively - NOT GOOD
+	 */
+
 	do {
-		if (*name == '-')
+		if (*name == '-' || *name == '.')
 			continue;
  skip:
 		if (isalnum(*name)
 		 || *name == '_'
-		 || *name == '.'
 		 || *name == '@'
-		 || (*name == '$' && !*(name + 1))
+		 || (*name == '$' && !name[1])
 		) {
 			continue;
 		}
