@@ -7,9 +7,10 @@
  * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
  */
 
-#include <sys/utsname.h>		/* for uname(2) */
-
 #include "libbb.h"
+/* After libbb.h, since it needs sys/types.h on some systems */
+#include <sys/utsname.h>  /* for uname(2) */
+
 
 /* Returns current kernel version encoded as major*65536 + minor*256 + patch,
  * so, for example,  to check if the kernel is greater than 2.2.11:
@@ -23,7 +24,7 @@ int FAST_FUNC get_linux_version_code(void)
 	int i, r;
 
 	if (uname(&name) == -1) {
-		bb_perror_msg("cannot get system information");
+		bb_perror_msg("can't get system information");
 		return 0;
 	}
 
