@@ -5,11 +5,10 @@
  *
  * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
  */
-
-#include <libbb.h>
+#include "libbb.h"
 
 /* returns a set with the flags not printed */
-int print_flags_separated(const int *masks, const char *labels, int flags, const char *separator)
+int FAST_FUNC print_flags_separated(const int *masks, const char *labels, int flags, const char *separator)
 {
 	const char *need_separator = NULL;
 	while (*labels) {
@@ -26,7 +25,7 @@ int print_flags_separated(const int *masks, const char *labels, int flags, const
 	return flags;
 }
 
-int print_flags(const masks_labels_t *ml, int flags)
+int FAST_FUNC print_flags(const masks_labels_t *ml, int flags)
 {
 	return print_flags_separated(ml->masks, ml->labels, flags, NULL);
 }
