@@ -17,11 +17,11 @@ int FAST_FUNC remove_file(const char *path, int flags)
 
 	if (lstat(path, &path_stat) < 0) {
 		if (errno != ENOENT) {
-			bb_perror_msg("cannot stat '%s'", path);
+			bb_perror_msg("can't stat '%s'", path);
 			return -1;
 		}
 		if (!(flags & FILEUTILS_FORCE)) {
-			bb_perror_msg("cannot remove '%s'", path);
+			bb_perror_msg("can't remove '%s'", path);
 			return -1;
 		}
 		return 0;
@@ -63,7 +63,7 @@ int FAST_FUNC remove_file(const char *path, int flags)
 		}
 
 		if (closedir(dp) < 0) {
-			bb_perror_msg("cannot close '%s'", path);
+			bb_perror_msg("can't close '%s'", path);
 			return -1;
 		}
 
@@ -74,7 +74,7 @@ int FAST_FUNC remove_file(const char *path, int flags)
 		}
 
 		if (rmdir(path) < 0) {
-			bb_perror_msg("cannot remove '%s'", path);
+			bb_perror_msg("can't remove '%s'", path);
 			return -1;
 		}
 
@@ -94,7 +94,7 @@ int FAST_FUNC remove_file(const char *path, int flags)
 	}
 
 	if (unlink(path) < 0) {
-		bb_perror_msg("cannot remove '%s'", path);
+		bb_perror_msg("can't remove '%s'", path);
 		return -1;
 	}
 
