@@ -54,9 +54,7 @@ void FAST_FUNC launch_helper(const char **argv)
 		+ (1 << SIGALRM)
 		, signal_handler);
 
-	G.helper_pid = vfork();
-	if (G.helper_pid < 0)
-		bb_perror_msg_and_die("vfork");
+	G.helper_pid = xvfork();
 
 	i = (!G.helper_pid) * 2; // for parent:0, for child:2
 	close(pipes[i + 1]); // 1 or 3 - closing one write end
@@ -67,8 +65,7 @@ void FAST_FUNC launch_helper(const char **argv)
 	if (!G.helper_pid) {
 		// child: try to execute connection helper
 		// NB: SIGCHLD & SIGALRM revert to SIG_DFL on exec
-		BB_EXECVP(*argv, (char **)argv);
-		_exit(127);
+		BB_EXECVP_or_die((char**)argv);
 	}
 
 	// parent
@@ -87,7 +84,7 @@ const FAST_FUNC char *command(const char *fmt, const char *param)
 		msg = xasprintf(fmt, param);
 		printf("%s\r\n", msg);
 	}
-	fflush(stdout);
+	fflush_all();
 	return msg;
 }
 
