@@ -7,6 +7,6 @@
 int bbconfig_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
 int bbconfig_main(int argc UNUSED_PARAM, char **argv UNUSED_PARAM)
 {
-	printf(bbconfig_config);
+	full_write1_str(bbconfig_config);
 	return 0;
 }
