@@ -70,7 +70,7 @@ static void resuse_end(pid_t pid, resource_t *resp)
 			return;
 		}
 	}
-	resp->elapsed_ms = (monotonic_us() / 1000) - resp->elapsed_ms;
+	resp->elapsed_ms = monotonic_ms() - resp->elapsed_ms;
 }
 
 static void printargv(char *const *argv)
@@ -367,20 +367,15 @@ static void summarize(const char *fmt, char **command, resource_t *resp)
    Put the statistics in *RESP.  */
 static void run_command(char *const *cmd, resource_t *resp)
 {
-	pid_t pid;			/* Pid of child.  */
+	pid_t pid;
 	void (*interrupt_signal)(int);
 	void (*quit_signal)(int);
 
-	resp->elapsed_ms = monotonic_us() / 1000;
-	pid = vfork();		/* Run CMD as child process.  */
-	if (pid < 0)
-		bb_perror_msg_and_die("fork");
-	if (pid == 0) {	/* If child.  */
-		/* Don't cast execvp arguments; that causes errors on some systems,
-		   versus merely warnings if the cast is left off.  */
-		BB_EXECVP(cmd[0], cmd);
-		xfunc_error_retval = (errno == ENOENT ? 127 : 126);
-		bb_error_msg_and_die("cannot run %s", cmd[0]);
+	resp->elapsed_ms = monotonic_ms();
+	pid = xvfork();
+	if (pid == 0) {
+		/* Child */
+		BB_EXECVP_or_die((char**)cmd);
 	}
 
 	/* Have signals kill the child but not self (if possible).  */
@@ -414,9 +409,7 @@ int time_main(int argc UNUSED_PARAM, char **argv)
 	run_command(argv, &res);
 
 	/* Cheat. printf's are shorter :) */
-	/* (but see bb_putchar() body for additional wrinkle!) */
-	xdup2(2, 1); /* just in case libc does something silly :( */
-	stdout = stderr;
+	xdup2(STDERR_FILENO, STDOUT_FILENO);
 	summarize(output_format, argv, &res);
 
 	if (WIFSTOPPED(res.waitstatus))
