@@ -7,8 +7,8 @@
  *
  * Licensed under GPLv2 or later, see file LICENSE in this tarball for details.
  */
-
 #include "libbb.h"
+#include "unicode.h"
 
 #if ENABLE_FEATURE_CHECK_TAINTED_MODULE
 enum {
@@ -46,6 +46,8 @@ int lsmod_main(int argc UNUSED_PARAM, char **argv UNUSED_PARAM)
 #if ENABLE_FEATURE_LSMOD_PRETTY_2_6_OUTPUT
 	char *token[4];
 	parser_t *parser = config_open("/proc/modules");
+	init_unicode();
+
 	printf("%-24sSize  Used by", "Module");
 	check_tainted();
 
@@ -58,7 +60,17 @@ int lsmod_main(int argc UNUSED_PARAM, char **argv UNUSED_PARAM)
 				token[3][strlen(token[3])-1] = '\0';
 			} else
 				token[3] = (char *) "";
+# if ENABLE_UNICODE_SUPPORT
+			{
+				uni_stat_t uni_stat;
+				char *uni_name = unicode_conv_to_printable(&uni_stat, token[0]);
+				unsigned pad_len = (uni_stat.unicode_width > 19) ? 0 : 19 - uni_stat.unicode_width;
+				printf("%s%*s %8s %2s %s\n", uni_name, pad_len, "", token[1], token[2], token[3]);
+				free(uni_name);
+			}
+# else
 			printf("%-19s %8s %2s %s\n", token[0], token[1], token[2], token[3]);
+# endif
 		}
 	} else {
 		while (config_read(parser, token, 4, 4, "# \t", PARSE_NORMAL & ~PARSE_GREEDY)) {
@@ -66,7 +78,17 @@ int lsmod_main(int argc UNUSED_PARAM, char **argv UNUSED_PARAM)
 			// or comma-separated list ended by comma
 			// so trimming the trailing char is just what we need!
 			token[3][strlen(token[3])-1] = '\0';
+# if ENABLE_UNICODE_SUPPORT
+			{
+				uni_stat_t uni_stat;
+				char *uni_name = unicode_conv_to_printable(&uni_stat, token[0]);
+				unsigned pad_len = (uni_stat.unicode_width > 19) ? 0 : 19 - uni_stat.unicode_width;
+				printf("%s%*s %8s %2s %s\n", uni_name, pad_len, "", token[1], token[2], token[3]);
+				free(uni_name);
+			}
+# else
 			printf("%-19s %8s %2s %s\n", token[0], token[1], token[2], token[3]);
+# endif
 		}
 	}
 	if (ENABLE_FEATURE_CLEAN_UP)
