@@ -9,10 +9,89 @@
 
 #include "libbb.h"
 
-#if ENABLE_DESKTOP
-#include "nc_bloaty.c"
+//config:config NC
+//config:	bool "nc"
+//config:	default y
+//config:	help
+//config:	  A simple Unix utility which reads and writes data across network
+//config:	  connections.
+//config:
+//config:config NC_SERVER
+//config:	bool "Netcat server options (-l)"
+//config:	default y
+//config:	depends on NC
+//config:	help
+//config:	  Allow netcat to act as a server.
+//config:
+//config:config NC_EXTRA
+//config:	bool "Netcat extensions (-eiw and filename)"
+//config:	default y
+//config:	depends on NC
+//config:	help
+//config:	  Add -e (support for executing the rest of the command line after
+//config:	  making or receiving a successful connection), -i (delay interval for
+//config:	  lines sent), -w (timeout for initial connection).
+//config:
+//config:config NC_110_COMPAT
+//config:	bool "Netcat 1.10 compatibility (+2.5k)"
+//config:	default n  # off specially for Rob
+//config:	depends on NC
+//config:	help
+//config:	  This option makes nc closely follow original nc-1.10.
+//config:	  The code is about 2.5k bigger. It enables
+//config:	  -s ADDR, -n, -u, -v, -o FILE, -z options, but loses
+//config:	  busybox-specific extensions: -f FILE and -ll.
+
+#if ENABLE_NC_110_COMPAT
+# include "nc_bloaty.c"
 #else
 
+//usage:#if !ENABLE_NC_110_COMPAT
+//usage:
+//usage:#if ENABLE_NC_SERVER || ENABLE_NC_EXTRA
+//usage:#define NC_OPTIONS_STR "\n\nOptions:"
+//usage:#else
+//usage:#define NC_OPTIONS_STR
+//usage:#endif
+//usage:
+//usage:#define nc_trivial_usage
+//usage:	IF_NC_EXTRA("[-iN] [-wN] ")IF_NC_SERVER("[-l] [-p PORT] ")
+//usage:       "["IF_NC_EXTRA("-f FILE|")"IPADDR PORT]"IF_NC_EXTRA(" [-e PROG]")
+//usage:#define nc_full_usage "\n\n"
+//usage:       "Open a pipe to IP:PORT" IF_NC_EXTRA(" or FILE")
+//usage:	NC_OPTIONS_STR
+//usage:	IF_NC_EXTRA(
+//usage:     "\n	-e PROG	Run PROG after connect"
+//usage:	IF_NC_SERVER(
+//usage:     "\n	-l	Listen mode, for inbound connects"
+//usage:	IF_NC_EXTRA(
+//usage:     "\n		(use -l twice with -e for persistent server)")
+//usage:     "\n	-p PORT	Local port"
+//usage:	)
+//usage:     "\n	-w SEC	Timeout for connect"
+//usage:     "\n	-i SEC	Delay interval for lines sent"
+//usage:     "\n	-f FILE	Use file (ala /dev/ttyS0) instead of network"
+//usage:	)
+//usage:
+//usage:#define nc_notes_usage ""
+//usage:	IF_NC_EXTRA(
+//usage:       "To use netcat as a terminal emulator on a serial port:\n\n"
+//usage:       "$ stty 115200 -F /dev/ttyS0\n"
+//usage:       "$ stty raw -echo -ctlecho && nc -f /dev/ttyS0\n"
+//usage:	)
+//usage:
+//usage:#define nc_example_usage
+//usage:       "$ nc foobar.somedomain.com 25\n"
+//usage:       "220 foobar ESMTP Exim 3.12 #1 Sat, 15 Apr 2000 00:03:02 -0600\n"
+//usage:       "help\n"
+//usage:       "214-Commands supported:\n"
+//usage:       "214-    HELO EHLO MAIL RCPT DATA AUTH\n"
+//usage:       "214     NOOP QUIT RSET HELP\n"
+//usage:       "quit\n"
+//usage:       "221 foobar closing connection\n"
+//usage:
+//usage:#endif
+
 /* Lots of small differences in features
  * when compared to "standard" nc
  */
@@ -39,7 +118,7 @@ int nc_main(int argc, char **argv)
 
 	if (ENABLE_NC_SERVER || ENABLE_NC_EXTRA) {
 		/* getopt32 is _almost_ usable:
-		** it cannot handle "... -e prog -prog-opt" */
+		** it cannot handle "... -e PROG -prog-opt" */
 		while ((opt = getopt(argc, argv,
 		        "" IF_NC_SERVER("lp:") IF_NC_EXTRA("w:i:f:e:") )) > 0
 		) {
@@ -57,7 +136,7 @@ int nc_main(int argc, char **argv)
 				/* We cannot just 'break'. We should let getopt finish.
 				** Or else we won't be able to find where
 				** 'host' and 'port' params are
-				** (think "nc -w 60 host port -e prog"). */
+				** (think "nc -w 60 host port -e PROG"). */
 				IF_NC_EXTRA(
 					char **p;
 					// +2: one for progname (optarg) and one for NULL
@@ -70,7 +149,7 @@ int nc_main(int argc, char **argv)
 				)
 				/* optind points to argv[arvc] (NULL) now.
 				** FIXME: we assume that getopt will not count options
-				** possibly present on "-e prog args" and will not
+				** possibly present on "-e PROG ARGS" and will not
 				** include them into final value of optind
 				** which is to be used ...  */
 			} else bb_show_usage();
@@ -137,10 +216,8 @@ int nc_main(int argc, char **argv)
 	if (execparam) {
 		pid_t pid;
 		/* With more than one -l, repeatedly act as server */
-		if (do_listen > 1 && (pid = vfork()) != 0) {
-			/* parent or error */
-			if (pid < 0)
-				bb_perror_msg_and_die("vfork");
+		if (do_listen > 1 && (pid = xvfork()) != 0) {
+			/* parent */
 			/* prevent zombies */
 			signal(SIGCHLD, SIG_IGN);
 			close(cfd);
