@@ -1,5 +1,7 @@
 /* vi: set sw=4 ts=4: */
-/* dhcpc.h */
+/*
+ * Licensed under GPLv2, see file LICENSE in this tarball for details.
+ */
 #ifndef UDHCP_DHCPC_H
 #define UDHCP_DHCPC_H 1
 
@@ -10,16 +12,16 @@ struct client_config_t {
 	char no_default_options;        /* Do not include default options in request */
 	IF_FEATURE_UDHCP_PORT(uint16_t port;)
 	int ifindex;                    /* Index number of the interface to use */
-	int verbose;
 	uint8_t opt_mask[256 / 8];      /* Bitmask of options to send (-O option) */
 	const char *interface;          /* The name of the interface to use */
 	char *pidfile;                  /* Optionally store the process ID */
 	const char *script;             /* User script to run at dhcp events */
+	struct option_set *options;     /* list of DHCP options to send to server */
 	uint8_t *clientid;              /* Optional client id to use */
 	uint8_t *vendorclass;           /* Optional vendor class-id to use */
 	uint8_t *hostname;              /* Optional hostname to use */
 	uint8_t *fqdn;                  /* Optional fully qualified domain name to use */
-};
+} FIX_ALIASING;
 
 /* server_config sits in 1st half of bb_common_bufsiz1 */
 #define client_config (*(struct client_config_t*)(&bb_common_bufsiz1[COMMON_BUFSIZE / 2]))
@@ -30,21 +32,6 @@ struct client_config_t {
 #define CLIENT_PORT 68
 #endif
 
-
-/*** clientpacket.h ***/
-
-uint32_t random_xid(void) FAST_FUNC;
-int send_discover(uint32_t xid, uint32_t requested) FAST_FUNC;
-int send_select(uint32_t xid, uint32_t server, uint32_t requested) FAST_FUNC;
-#if ENABLE_FEATURE_UDHCPC_ARPING
-int send_decline(uint32_t xid, uint32_t server, uint32_t requested) FAST_FUNC;
-#endif
-int send_renew(uint32_t xid, uint32_t server, uint32_t ciaddr) FAST_FUNC;
-int send_renew(uint32_t xid, uint32_t server, uint32_t ciaddr) FAST_FUNC;
-int send_release(uint32_t server, uint32_t ciaddr) FAST_FUNC;
-
-int udhcp_recv_raw_packet(struct dhcp_packet *dhcp_pkt, int fd) FAST_FUNC;
-
 POP_SAVED_FUNCTION_VISIBILITY
 
 #endif
