@@ -7,11 +7,11 @@
 #include "libbb.h"
 
 int load_policy_main(int argc, char **argv) MAIN_EXTERNALLY_VISIBLE;
-int load_policy_main(int argc, char **argv UNUSED_PARAM)
+int load_policy_main(int argc UNUSED_PARAM, char **argv UNUSED_PARAM)
 {
 	int rc;
 
-	if (argc != 1) {
+	if (argv[1]) {
 		bb_show_usage();
 	}
 
