@@ -9,9 +9,25 @@
 #include "libbb.h"
 
 #include <linux/input.h>
-#ifndef SW_RFKILL_ALL
-# define SW_RFKILL_ALL 3
+#ifndef EV_SW
+# define EV_SW         0x05
 #endif
+#ifndef EV_KEY
+# define EV_KEY        0x01
+#endif
+#ifndef SW_LID
+# define SW_LID        0x00
+#endif
+#ifndef SW_RFKILL_ALL
+# define SW_RFKILL_ALL 0x03
+#endif
+#ifndef KEY_POWER
+# define KEY_POWER     116     /* SC System Power Down */
+#endif
+#ifndef KEY_SLEEP
+# define KEY_SLEEP     142     /* SC System Sleep */
+#endif
+
 
 /*
  * acpid listens to ACPI events coming either in textual form
@@ -74,6 +90,7 @@ int acpid_main(int argc, char **argv)
 	}
 
 	argv += optind;
+	argc -= optind;
 
 	// goto configuration directory
 	xchdir(opt_conf);
@@ -102,7 +119,7 @@ int acpid_main(int argc, char **argv)
 	// evdev files given, use evdev interface
 
 	// open event devices
-	pfd = xzalloc(sizeof(*pfd) * (argc - optind));
+	pfd = xzalloc(sizeof(*pfd) * argc);
 	nfd = 0;
 	while (*argv) {
 		pfd[nfd].fd = open_or_warn(*argv++, O_RDONLY | O_NONBLOCK);
