@@ -65,8 +65,8 @@ struct xbsd_disklabel {
 	int16_t    d_type;                 /* drive type */
 	int16_t    d_subtype;              /* controller/d_type specific */
 	char       d_typename[16];         /* type name, e.g. "eagle" */
-	char       d_packname[16];                 /* pack identifier */
-			/* disk geometry: */
+	char       d_packname[16];         /* pack identifier */
+	/* disk geometry: */
 	uint32_t   d_secsize;              /* # of bytes per sector */
 	uint32_t   d_nsectors;             /* # of data sectors per track */
 	uint32_t   d_ntracks;              /* # of tracks per cylinder */
@@ -87,7 +87,7 @@ struct xbsd_disklabel {
 	 */
 	uint32_t   d_acylinders;           /* # of alt. cylinders per unit */
 
-			/* hardware characteristics: */
+	/* hardware characteristics: */
 	/*
 	 * d_interleave, d_trackskew and d_cylskew describe perturbations
 	 * in the media format used to compensate for a slow controller.
@@ -117,11 +117,11 @@ struct xbsd_disklabel {
 	uint32_t   d_spare[NSPARE];        /* reserved for future use */
 	uint32_t   d_magic2;               /* the magic number (again) */
 	uint16_t   d_checksum;             /* xor of data incl. partitions */
-			/* filesystem and partition information: */
+	/* filesystem and partition information: */
 	uint16_t   d_npartitions;          /* number of partitions in following */
 	uint32_t   d_bbsize;               /* size of boot area at sn0, bytes */
 	uint32_t   d_sbsize;               /* max size of fs superblock, bytes */
-	struct xbsd_partition    {      /* the partition table */
+	struct xbsd_partition { /* the partition table */
 		uint32_t   p_size;         /* number of sectors in partition */
 		uint32_t   p_offset;       /* starting sector */
 		uint32_t   p_fsize;        /* filesystem basic fragment size */
@@ -367,7 +367,7 @@ bsd_select(void)
 					partname(disk_device, t+1, 0));
 				return;
 			}
-				printf("Reading disklabel of %s at sector %d\n",
+				printf("Reading disklabel of %s at sector %u\n",
 					partname(disk_device, t+1, 0), ss + BSD_LABELSECTOR);
 			if (xbsd_readlabel(xbsd_part) == 0)
 				if (xbsd_create_disklabel() == 0)
@@ -510,7 +510,7 @@ xbsd_print_disklabel(int show_all)
 		if ((unsigned) lp->d_type < ARRAY_SIZE(xbsd_dktypenames)-1)
 			printf("type: %s\n", xbsd_dktypenames[lp->d_type]);
 		else
-			printf("type: %d\n", lp->d_type);
+			printf("type: %u\n", lp->d_type);
 		printf("disk: %.*s\n", (int) sizeof(lp->d_typename), lp->d_typename);
 		printf("label: %.*s\n", (int) sizeof(lp->d_packname), lp->d_packname);
 		printf("flags: ");
@@ -518,18 +518,18 @@ xbsd_print_disklabel(int show_all)
 		bb_putchar('\n');
 		/* On various machines the fields of *lp are short/int/long */
 		/* In order to avoid problems, we cast them all to long. */
-		printf("bytes/sector: %ld\n", (long) lp->d_secsize);
-		printf("sectors/track: %ld\n", (long) lp->d_nsectors);
-		printf("tracks/cylinder: %ld\n", (long) lp->d_ntracks);
-		printf("sectors/cylinder: %ld\n", (long) lp->d_secpercyl);
-		printf("cylinders: %ld\n", (long) lp->d_ncylinders);
-		printf("rpm: %d\n", lp->d_rpm);
-		printf("interleave: %d\n", lp->d_interleave);
-		printf("trackskew: %d\n", lp->d_trackskew);
-		printf("cylinderskew: %d\n", lp->d_cylskew);
-		printf("headswitch: %ld\t\t# milliseconds\n",
+		printf("bytes/sector: %lu\n", (long) lp->d_secsize);
+		printf("sectors/track: %lu\n", (long) lp->d_nsectors);
+		printf("tracks/cylinder: %lu\n", (long) lp->d_ntracks);
+		printf("sectors/cylinder: %lu\n", (long) lp->d_secpercyl);
+		printf("cylinders: %lu\n", (long) lp->d_ncylinders);
+		printf("rpm: %u\n", lp->d_rpm);
+		printf("interleave: %u\n", lp->d_interleave);
+		printf("trackskew: %u\n", lp->d_trackskew);
+		printf("cylinderskew: %u\n", lp->d_cylskew);
+		printf("headswitch: %lu\t\t# milliseconds\n",
 			(long) lp->d_headswitch);
-		printf("track-to-track seek: %ld\t# milliseconds\n",
+		printf("track-to-track seek: %lu\t# milliseconds\n",
 			(long) lp->d_trkseek);
 		printf("drivedata: ");
 		for (i = NDDATA - 1; i >= 0; i--)
@@ -538,25 +538,25 @@ xbsd_print_disklabel(int show_all)
 		if (i < 0)
 			i = 0;
 		for (j = 0; j <= i; j++)
-			printf("%ld ", (long) lp->d_drivedata[j]);
+			printf("%lu ", (long) lp->d_drivedata[j]);
 	}
-	printf("\n%d partitions:\n", lp->d_npartitions);
+	printf("\n%u partitions:\n", lp->d_npartitions);
 	printf("#       start       end      size     fstype   [fsize bsize   cpg]\n");
 	pp = lp->d_partitions;
 	for (i = 0; i < lp->d_npartitions; i++, pp++) {
 		if (pp->p_size) {
 			if (display_in_cyl_units && lp->d_secpercyl) {
-				printf("  %c: %8ld%c %8ld%c %8ld%c  ",
+				printf("  %c: %8lu%c %8lu%c %8lu%c  ",
 					'a' + i,
-					(long) pp->p_offset / lp->d_secpercyl + 1,
+					(unsigned long) pp->p_offset / lp->d_secpercyl + 1,
 					(pp->p_offset % lp->d_secpercyl) ? '*' : ' ',
-					(long) (pp->p_offset + pp->p_size + lp->d_secpercyl - 1) / lp->d_secpercyl,
+					(unsigned long) (pp->p_offset + pp->p_size + lp->d_secpercyl - 1) / lp->d_secpercyl,
 					((pp->p_offset + pp->p_size) % lp->d_secpercyl) ? '*' : ' ',
 					(long) pp->p_size / lp->d_secpercyl,
 					(pp->p_size % lp->d_secpercyl) ? '*' : ' '
 				);
 			} else {
-				printf("  %c: %8ld  %8ld  %8ld   ",
+				printf("  %c: %8lu  %8lu  %8lu   ",
 					'a' + i,
 					(long) pp->p_offset,
 					(long) pp->p_offset + pp->p_size - 1,
@@ -571,11 +571,11 @@ xbsd_print_disklabel(int show_all)
 
 			switch (pp->p_fstype) {
 			case BSD_FS_UNUSED:
-				printf("    %5ld %5ld %5.5s ",
+				printf("    %5lu %5lu %5.5s ",
 					(long) pp->p_fsize, (long) pp->p_fsize * pp->p_frag, "");
 				break;
 			case BSD_FS_BSDFFS:
-				printf("    %5ld %5ld %5d ",
+				printf("    %5lu %5lu %5u ",
 					(long) pp->p_fsize, (long) pp->p_fsize * pp->p_frag, pp->p_cpg);
 				break;
 			default:
@@ -615,7 +615,7 @@ xbsd_create_disklabel(void)
 
 	while (1) {
 		c = read_nonempty("Do you want to create a disklabel? (y/n) ");
-		if (c == 'y' || c == 'Y') {
+		if ((c|0x20) == 'y') {
 			if (xbsd_initlabel(
 #if defined(__alpha__) || defined(__powerpc__) || defined(__hppa__) || \
 	defined(__s390__) || defined(__s390x__)
@@ -629,7 +629,7 @@ xbsd_create_disklabel(void)
 			}
 			return 0;
 		}
-		if (c == 'n')
+		if ((c|0x20) == 'n')
 			return 0;
 	}
 }
@@ -637,7 +637,7 @@ xbsd_create_disklabel(void)
 static int
 edit_int(int def, const char *mesg)
 {
-	mesg = xasprintf("%s (%d): ", mesg, def);
+	mesg = xasprintf("%s (%u): ", mesg, def);
 	do {
 		if (!read_line(mesg))
 			goto ret;
@@ -950,7 +950,7 @@ xbsd_readlabel(struct partition *p)
 	}
 
 	if (d->d_npartitions > BSD_MAXPARTITIONS)
-		printf("Warning: too many partitions (%d, maximum is %d)\n",
+		printf("Warning: too many partitions (%u, maximum is %u)\n",
 			d->d_npartitions, BSD_MAXPARTITIONS);
 	return 1;
 }
@@ -964,6 +964,7 @@ xbsd_writelabel(struct partition *p)
 #if !defined(__alpha__) && !defined(__powerpc__) && !defined(__hppa__)
 	sector = get_start_sect(p) + BSD_LABELSECTOR;
 #else
+	(void)p; /* silence warning */
 	sector = BSD_LABELSECTOR;
 #endif
 
