@@ -24,10 +24,10 @@ struct romfs_super {
 	uint8_t magic[8];
 	uint32_t size;
 	uint32_t checksum;
-	uint8_t name[0];
-} __attribute__((__packed__));
+	uint8_t name[];
+} PACKED;
 
-int volume_id_probe_romfs(struct volume_id *id /*,uint64_t off*/)
+int FAST_FUNC volume_id_probe_romfs(struct volume_id *id /*,uint64_t off*/)
 {
 #define off ((uint64_t)0)
 	struct romfs_super *rfs;
